precision mediump float;

uniform sampler2D      sTexture;
uniform mediump vec2   MinTexCoord;
uniform mediump vec2   MaxTexCoord;
uniform mediump float  TimePortion;
uniform bool           ReverseEffect;

uniform bool           UseColor;
uniform mediump vec3   Color;

varying mediump vec2   TexCoord;

void main()
{
    float scale = 6.0;
    float t = (1.0 + scale) * TimePortion; // t is in [0; 7]
    
    vec2 center = mix(MinTexCoord, MaxTexCoord, 0.5);
    float x = abs((TexCoord.x - center.x) / (MaxTexCoord.x - center.x)); // x is in [0; 1]
    float y = abs((TexCoord.y - center.y) / (MaxTexCoord.y - center.y)); // y is in [0; 1]
    float a = scale * max(x, y); // a is in [0; 6]
    
    float alpha = clamp(t - a, 0.0, 1.0);
    
    // check reverse alpha
    float reverseAlpha = float(ReverseEffect); // reverse alpha is 0 or 1
    alpha = reverseAlpha * (1.0 - alpha) + (1.0 - reverseAlpha) * alpha;
    
    ////////////////////////////
    // Apply color
    vec4 textureColor = texture2D(sTexture, TexCoord);
    
    float useColor = float(UseColor); // useColor is 0 or 1
    
    // The following code line says that:
    // color = textureColor, if UseColor is false
    // color = vec4(Color * textureColor.a, textureColor.a), if UseColor is true
    vec4 color = (1.0 - useColor) * textureColor + useColor * vec4(Color * textureColor.a, textureColor.a);
    ////////////////////////////
    
    gl_FragColor = color * alpha;
}
