precision mediump float;

uniform sampler2D       sTexture;
uniform mediump float   TimePortion;
uniform bool            ReverseEffect;

uniform bool            UseColor;
uniform mediump vec3    Color;

varying mediump vec2    TexCoord;

void main()
{
    float alpha = TimePortion;
    
    // check reverse alpha
    float reverseAlpha = float(ReverseEffect); // reverse alpha is 0 or 1
    alpha = reverseAlpha * (1.0 - alpha) + (1.0 - reverseAlpha) * alpha;
 
    ////////////////////////////
    // Apply color
    vec4 textureColor = texture2D(sTexture, TexCoord);
    
    float useColor = float(UseColor); // useColor is 0 or 1
    
    // The following code line says that:
    // color = textureColor, if UseColor is false
    // color = vec4(Color * textureColor.a, textureColor.a), if UseColor is true
    vec4 color = (1.0 - useColor) * textureColor + useColor * vec4(Color * textureColor.a, textureColor.a);
    ////////////////////////////
    
    gl_FragColor = color * alpha; // because we are using premultiplied alpha textures
}
