precision mediump float;

uniform sampler2D  sTexture;
uniform mediump vec2  TextureSize;
uniform mediump float TimeParameter;

varying mediump vec2   TexCoord;

void main()
{
    gl_FragColor = texture2D(sTexture, TexCoord);
    
    float diagonal = length(TextureSize);
    
    float deltaT = 2.0 * TimeParameter * diagonal; // TimeParameter is in [0.0, 1.0]
    vec2 center = vec2(0.0, TextureSize.y);
    float dist = distance(center, TexCoord);
    vec2 direction = (center - TexCoord);
    
    gl_FragColor.a = pow(clamp((deltaT - dist)/diagonal, 0.0, 1.0), 2.0);
    
    float radius = clamp(1.0 - dist/diagonal, 0.0, 1.0);
    float sine = sin((dist + deltaT) * 20.0);
    float offset = radius * sine * 0.05 * (1.0 - TimeParameter);
    
    vec2 uv = TexCoord + direction * offset;
    float light = 1.0 + offset * 8.0;
    gl_FragColor.rgb = texture2D(sTexture, uv).xyz * light;
}
