precision mediump float;

uniform sampler2D      sTexture;
uniform mediump vec2   MinTexCoord;
uniform mediump vec2   MaxTexCoord;
uniform mediump float  TimePortion;
uniform bool           ReverseEffect;

uniform bool           UseColor;
uniform mediump vec3   Color;

varying mediump vec2   TexCoord;

void main()
{
    // apply reverse effect
    float reverse = float(ReverseEffect); // reverse is 0 or 1
    float time = reverse * (1.0 - TimePortion) + (1.0 - reverse) * TimePortion;
    
    float scale = 6.0;
    float t = (1.0 + scale) * time; // t is in [0; 7]
    float y = scale * (MaxTexCoord.y - TexCoord.y) / (MaxTexCoord.y - MinTexCoord.y); // y is in [0; 6]
    
    float alpha = clamp(t - y, 0.0, 1.0);
    
    // clip coordinates that are out of texture
    bool inTexture = (sign(MaxTexCoord.x - TexCoord.x) != -1.0) &&
                     (sign(TexCoord.x - MinTexCoord.x) != -1.0) &&
                     (sign(MaxTexCoord.y - TexCoord.y) != -1.0) &&
                     (sign(TexCoord.y - MinTexCoord.y) != -1.0);
    alpha = alpha * float(inTexture);
    
    ////////////////////////////
    // Apply color
    vec4 textureColor = texture2D(sTexture, TexCoord);
    
    float useColor = float(UseColor); // useColor is 0 or 1
    
    // The following code line says that:
    // color = textureColor, if UseColor is false
    // color = vec4(Color * textureColor.a, textureColor.a), if UseColor is true
    vec4 color = (1.0 - useColor) * textureColor + useColor * vec4(Color * textureColor.a, textureColor.a);
    ////////////////////////////
    
    gl_FragColor = color * alpha;
}
