attribute highp   vec3  inVertex;
attribute mediump vec2  inTexCoord;

uniform highp   mat4   MVPMatrix;
uniform mediump vec2   MinTexCoord;
uniform mediump vec2   MaxTexCoord;
uniform mediump float  TimePortion;
uniform bool ReverseEffect;

varying mediump vec2   TexCoord;

void main()
{
    // NOTE: Here we rely on the fact that our object is zero centered
    float scaleFactor = 2.0 / 6.0;
    vec3 position = inVertex * vec3(1.0 + scaleFactor, 1.0 + scaleFactor, 1.0);
    gl_Position = MVPMatrix * vec4(position, 1.0);
    
    vec2 texSize = MaxTexCoord - MinTexCoord;
    TexCoord = inTexCoord * (1.0 + scaleFactor) - 0.5 * scaleFactor * texSize;
    
    // apply reverse effect
    float reverse = float(ReverseEffect); // reverse is 0 or 1
    float time = reverse * (1.0 - TimePortion) + (1.0 - reverse) * TimePortion;
    
    TexCoord = TexCoord + vec2(texSize.x, 0.0) * (1.0 + 0.5 * scaleFactor) * (1.0 - time);
}
