precision mediump float;

uniform sampler2D      sTexture;
uniform mediump vec2   MinTexCoord;
uniform mediump vec2   MaxTexCoord;
uniform mediump float  TimePortion;
uniform bool           ReverseEffect;

uniform bool           UseColor;
uniform mediump vec3   Color;

varying mediump vec2   TexCoord;

void main()
{
    float scale = 16.0;
    float x = scale * (MaxTexCoord.x - TexCoord.x) / (MaxTexCoord.x - MinTexCoord.x); // x is in [0; 16]
    x = floor(x); // here almost all (or even all) x values will be in [0; 15]
    x = clamp(x, 0.0, scale - 1.0); // x is in [0; 15]
    
    float y = scale * (TexCoord.y - MinTexCoord.y) / (MaxTexCoord.y - MinTexCoord.y); // y is in [0; 16]
    y = floor(y); // here almost all (or even all) y values will be in [0; 15]
    y = clamp(y, 0.0, scale - 1.0); // y is in [0; 15]
    
    // we should shift according to "column number"
    y += x; // y is in [0; 30]
    
    // check reverse alpha
    float reverseEffect = float(ReverseEffect); // reverse alpha is 0 or 1
    float t = reverseEffect * (1.0 - TimePortion) + (1.0 - reverseEffect) * TimePortion; // t is TimePortion or (1 - TimePortion)
    
    // we want see 4 parts animation (in row) at a time
    float simultaneous = 4.0;
    t = (2.0 * (scale - 1.0) + simultaneous) * t; // t is in [0; 34]
    float alpha = clamp(t - y, 0.0, simultaneous); // alpha is in [0; 4]
    alpha /= simultaneous; // alpha is in [0; 1]
    
    ////////////////////////////
    // Apply color
    vec4 textureColor = texture2D(sTexture, TexCoord);
    
    float useColor = float(UseColor); // useColor is 0 or 1
    
    // The following code line says that:
    // color = textureColor, if UseColor is false
    // color = vec4(Color * textureColor.a, textureColor.a), if UseColor is true
    vec4 color = (1.0 - useColor) * textureColor + useColor * vec4(Color * textureColor.a, textureColor.a);
    ////////////////////////////
    
    gl_FragColor = color * alpha;
}
